@@ -12,8 +12,8 @@ use fields qw(loop proto dst fds tids cb timeout_timer);
 use Net::SIP::Util ':all';
 use Net::SIP::Packet;
 use Net::SIP::Debug;
+use Net::SIP::Dispatcher::Eventloop;
 use Socket qw(SOL_SOCKET SO_ERROR);
-use Errno 'EINVAL';
 
 # RFC does not specify some fixed limit for the SIP header and body so we have
 # to make up some limits we think are useful.
@@ -23,9 +23,10 @@ my $MAX_SIP_BODY   = 2**16;   # 64k body
 # how many requests we can associate with a socket at the same time
 my $MAX_TIDLIST = 30;
 
-my $MIN_EXPIRE = 5;      # wait at least this time before closing on inactivity
-my $MAX_EXPIRE = 120;    # wait at most this time
-my $CONNECT_TIMEOUT = 5; # max time for TCP connect
+my $MIN_EXPIRE = 15;      # wait at least this time before closing on inactivity
+my $MAX_EXPIRE = 120;     # wait at most this time
+my $CONNECT_TIMEOUT = 10; # max time for TCP connect
+my $TCP_READSIZE = 2**16; # size of TCP read
 
 sub import {
     my %m = (
@@ -35,6 +36,7 @@ sub import {
 	MIN_EXPIRE      => \$MIN_EXPIRE,
 	MAX_EXPIRE      => \$MAX_EXPIRE,
 	CONNECT_TIMEOUT => \$CONNECT_TIMEOUT,
+	TCP_READSIZE    => \$TCP_READSIZE,
     );
     for(my $i=1;$i<@_;$i+=2) {
 	my $ref = $m{$_[$i]} or die "no such config key '$_[$i]'";
@@ -256,18 +258,22 @@ sub _error {
 
 {
     my %type2cb = (
+	# unconnected UDP socket: receive and send
 	udp_m  => sub { 
 	    my Net::SIP::SocketPool $self = shift;
 	    return $self->{dst}
 		? sub { _handle_read_udp(@_,1) }
 		: sub { _handle_read_udp(@_) }
 	},
+	# connected UDP socket: receive and send with fixed peer
 	udp_co => sub { 
 	    return \&_handle_read_udp 
 	},
+	# unconnected TCP socket: listen, accept and create tcp_co
 	tcp_m  => sub { 
 	    return \&_handle_read_tcp_m 
 	},
+	# connected TCP socket: receive and send with fixed peer
 	tcp_co => sub { 
 	    my (undef,$fd) = @_;
 	    my $from = getpeername($fd);
@@ -277,9 +283,10 @@ sub _error {
     sub _addreader2loop {
 	my Net::SIP::SocketPool $self = shift;
 	my $fo = shift;
+	# proto_co: connected socket, proto_m: (unconnected) master socket
 	my $type = $self->{proto} . ($fo->{peer} ? '_co':'_m');
-	$self->{loop}->addFD($fo->{fd}, 0, [
-	    $type2cb{$type}($self,$fo->{fd}) || die, 
+	$self->{loop}->addFD($fo->{fd}, EV_READ, [
+	    $type2cb{$type}($self,$fo->{fd}),
 	    $self
 	]);
     }
@@ -352,10 +359,13 @@ sub _handle_read_tcp_m {
 sub _handle_read_tcp_co {
     my Net::SIP::SocketPool $self = shift;
     my ($fd,$from) = @_;
+    my $fo = $self->{fds}{ fileno($fd) } or die "no fd for read";
+
+    $DEBUG && $fo->{rbuf} ne '' && DEBUG(20,
+	"continue reading SIP packet, offset=%d",length($fo->{rbuf}));
 
-    my $fo = $self->{fds}{ fileno($fd) } or die;
     retry:
-    my $n = sysread($fd, $fo->{rbuf}, 2**16, length($fo->{rbuf}));
+    my $n = sysread($fd, $fo->{rbuf}, $TCP_READSIZE, length($fo->{rbuf}));
     if (!defined $n) {
 	goto retry if $!{EINTR};
 	return if $!{EAGAIN} || $!{EWOULDBLOCK};
@@ -368,6 +378,7 @@ sub _handle_read_tcp_co {
 	return $self->_del_socket($fo);
     }
 
+    process_packet:
     # ignore any leading \r\n according to RFC 3261 7.5
     $fo->{rbuf} =~s{\A(\r\n)+}{};
 
@@ -401,8 +412,9 @@ sub _handle_read_tcp_co {
 	    "drop packet from %s since SIP body is too big: %d>%d",
 	    ip_sockaddr2string($from), $clen, $MAX_SIP_BODY);
     }
-    if ($hdrpos + $clen < length($fo->{rbuf})) {
-	$DEBUG && DEBUG(20,"need more data for SIP body");
+    if ($hdrpos + $clen > length($fo->{rbuf})) {
+	$DEBUG && DEBUG(20,"need %d more bytes for SIP body",
+	    $hdrpos + $clen - length($fo->{rbuf}));
 	return;
     }
 
@@ -426,6 +438,9 @@ sub _handle_read_tcp_co {
     $fo->{didit} = $self->{loop}->looptime if $self->{loop};
     invoke_callback($self->{cb},$pkt,
 	[$self->{proto}, ip_sockaddr2parts($from)]);
+
+    # continue with processing any remaining data in the buffer
+    goto process_packet if $fo->{rbuf} ne '';
 }
 
 sub _tcp_connect {
@@ -446,7 +461,7 @@ sub _tcp_connect {
 	if ($!{EALREADY} || $!{EINPROGRESS}) {
 	    # insert write handler
 	    $DEBUG && DEBUG(100,"tcp connect: add write handler for async connect");
-	    $self->{loop}->addFD($fo->{fd},1, 
+	    $self->{loop}->addFD($fo->{fd}, EV_WRITE,
 		[ \&_tcp_connect, $self,$fo,$peer,$callback ]);
 	    return;
 	}
@@ -462,7 +477,7 @@ sub _tcp_connect {
 	# we are called from loop and hopefully async connect was succesful:
 	# use getsockopt to check
 	my $err = getsockopt($xxfd, SOL_SOCKET, SO_ERROR);
-	$err = $err ? unpack('i',$err) : EINVAL;
+	$err = $err ? unpack('i',$err) : $!;
 	if ($err) {
 	    # connection failed
 	    $! = $err;
@@ -473,7 +488,8 @@ sub _tcp_connect {
 	}
 
 	# connect done: remove write handler
-	$self->{loop}->delFD($xxfd,1);
+	$self->{loop}->delFD($xxfd, EV_WRITE);
+	delete $fo->{inside_connect};
     }
 
     _addreader2loop($self,$fo);
@@ -499,7 +515,7 @@ sub _tcp_send {
 	if ($!{EAGAIN} || $!{EWOULDBLOCK}) {
 	    return if $xxfd; # called from loop: write handler already set up
 	    # insert write handler
-	    $self->{loop}->addFD($fo->{fd},1, 
+	    $self->{loop}->addFD($fo->{fd}, EV_WRITE,
 		[ \&_tcp_send, $self,$fo,$callback ]);
 	    return;
 	}
@@ -513,7 +529,7 @@ sub _tcp_send {
 
     # write done: remove write handler if we are called from loop
     $DEBUG && DEBUG(90,"everything has been sent");
-    $self->{loop}->delFD($xxfd,1) if $xxfd;
+    $self->{loop}->delFD($xxfd, EV_WRITE) if $xxfd;
 
     # signal success via callback
     invoke_callback($callback,0)
